@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from workalendar.core import WesternCalendar, ChristianMixin
-from workalendar.registry import iso_register
+from datetime import date
+from ..core import WesternCalendar, ChristianMixin
+from ..registry_tools import iso_register
 
 
 @iso_register('LU')
@@ -19,3 +18,10 @@ class Luxembourg(WesternCalendar, ChristianMixin):
         (5, 1, "Labour Day"),
         (6, 23, "Luxembourg National Holiday"),
     )
+
+    def get_fixed_holidays(self, year):
+        days = super().get_fixed_holidays(year)
+        if year > 2018:
+            days.append((date(year, 5, 9), "Europe Day"))
+
+        return days
